﻿Type=Class
Version=2.00
@EndOfDesignText@
'Class module cTabHost v1.0
'Author: Ricky Dalley, user Ricky D on the B4A forums
'
'This class is a TabHost control.
'
'I created it because I found a very annoying "feature" of the built-in TabHost
'where if you had an editable field like edittext or autocompleteeditext the keyboard would automatically
'show no matter what I tried to do.
'
'This one is highly customizable.
Sub Class_Globals
	
	Private mMeObj As Object
	Private mAct As Activity
	Private mTabChangedCallBack As String
	
	Private mTabs As Map
	Private mCurrentTab As Int 
	Private mMOVEMENTTHRESHOLD As Int : mMOVEMENTTHRESHOLD = 50
	Private mStartX As Int
	
	Private mButtons As Map
End Sub

'Initializes the object.
'
'MeObj is the calling Activity's Me object
'Act is the calling Activity where our tabhost will be placed
'TabChangedCallBack is the sub name in the calling Activity which receives the callback from TabChanged. Set it to "" to do nothing
'Tabs is a Map with Key = TabName and Value = an initialized Panel. Pass Null if you don't want to add prepared panels
'
'Example: <code>Dim m As Map
'm.Initialize
'Dim p As Panel
'p.Initialize("TabContent")
'p.LoadLayout("yourlayout.bal")
'm.Put("Page 1", p)
'Dim p As Panel
'p.Initialize("TabContent")
'p.LoadLayout("yourlayout2.bal")
'm.Put("Page 2", p)
'th.Initialize(Me, Activity, "TabChanged", m)</code>
Public Sub Initialize(MeObj As Object, Act As Activity, TabChangedCallBack As String, Tabs As Map)
	mMeObj = MeObj
	mAct = Act
	mTabChangedCallBack = TabChangedCallBack
	
	mTabs.Initialize
	mButtons.Initialize 
	mCurrentTab = -1
	
	If Not(Tabs.IsInitialized) OR Tabs.Size=0 Then
	Else
		'Add the tabs in Tabs
		For i=0 To Tabs.Size-1
			Dim TabName As String
			Dim p As Panel
			TabName = Tabs.GetKeyAt(i)
			p = Tabs.GetValueAt(i)
			AddTab(TabName, p)
		Next
	End If
End Sub

'Add a new tab to the tabhost
'Will throw an error if the tab already exists
'
'TabName is the tab you want to add
'p is the tab content. You can pass a prepared panel here or Null
'
'Example: <code>
'th.AddTab("Page 1", Null)	'passing Null will force the tabhost to create the content panel for us
' 'or
'Dim p As Panel
'p.Initialize("TabContent")
'p.LoadLayout("yourlayoutfile.bal")
'p.Color = Colors.Blue
'th.AddTab("Page 1", p)</code>
Public Sub AddTab(TabName As String, p As Panel)
	If mTabs.ContainsKey(TabName) Then
		Msgbox("Can't add the tab because it already exists", "Unable to add")
	Else
		If Not(p.IsInitialized) Then
			Dim pnl As Panel
			pnl.Initialize("Panel")
			pnl.Color = Colors.Gray 
			mTabs.Put(TabName, pnl)
		Else
			Dim pnl As Panel
			pnl = p
			mTabs.Put(TabName, pnl)
		End If
		
		'now add it's tab header - a button, and re-arrange the header lineup
		SetupTabHeaders(TabName, mAct)
		
		pnl.Tag = TabName
		'now add the panel to the activity
		mAct.AddView(pnl, 0, 80, 100%x, 100%y)
		mCurrentTab = 0
	End If
End Sub

'Set the visibility of a tab
'TabName is the tab you want to hide or show
'Visibility as the action you want to do. Set it to True to make the tab visible, False to hide it
'
'Example: <code>th.setTabVisibility("Page 1", False)</code>
Public Sub setTabVisibility(TabName As String, Visibility As Boolean )
	If mTabs.ContainsKey(TabName) Then
		Dim b As Button 
		b = mButtons.Get(TabName)
		b.Visible = Visibility
		Dim p As Panel
		p = mTabs.Get(TabName)
		p.Visible = Visibility
	Else
		Msgbox("Error in setTabVisibility - TabName " & TabName & " doesn't exist in the tabhost", "Error")
	End If
End Sub

'Remove the tab from the tabhost
'
'TabName is the tab you want to remove
'Throws error if tab doesn't exist in the tabhost
'
'Example: <code>
'th.RemoveTab("Page 1")
'th.ShowTab("Page 2")</code>
Public Sub RemoveTab(TabName As String)
	If mTabs.ContainsKey(TabName) Then
		'try to remove the Tab Header button
		If mButtons.ContainsKey(TabName) Then
			'remove it from the activity
			Dim b1 As Button 
			b1 = mButtons.Get(TabName)
			For i=mAct.NumberOfViews-1 To 0 Step -1
				Dim v As View 
				v = mAct.GetView(i)
				If v Is Button Then
					Dim b As Button 
					b = v
					If b=b1 Then
						b.RemoveView
						'now remove it from the internal map
						mButtons.Remove(TabName)
						Exit
					End If
				End If
			Next
		End If
		
		'now remove the tab content panel
		Dim p As Panel
		p = mTabs.Get(TabName)
		For i=p.NumberOfViews-1 To 0 Step -1
			Dim v As View
			v = p.GetView(i)
			v.RemoveView 
		Next
		'now remove it from the activity
		For i=mAct.NumberOfViews-1 To 0 Step - 1
			Dim v As View 
			v = mAct.GetView(i)
			If v Is Panel Then
				Dim p As Panel
				p = v
				If p.Tag=TabName Then
					v.RemoveView 
					mTabs.Remove(TabName)
					Exit
				End If
			End If
		Next
		
		ShowTabHeaders(mAct)
	Else
		Msgbox("Error in RemoveTab. Tab " & TabName & " doesn't exist in the TabHost", "Error")
	End If
End Sub

Private Sub ShowTabHeaders(act As Activity)
	'Remove the existing Tabs buttons
	For i=act.NumberOfViews-1 To 0 Step -1
		Dim v As View
		v = mAct.GetView(i)
		If v Is Button Then
			v.RemoveView
		End If
	Next
	
	'now re-add the buttons
	Dim width As Int
	width = mAct.width / mButtons.Size 
	For i=0 To mButtons.Size-1
		Dim b As Button
		b = mButtons.GetValueAt(i)
		act.AddView(b, width*i, 0, width, 80)
	Next
End Sub

Private Sub SetupTabHeaders(TabName, act As Activity)
	Dim b As Button 
	b.Initialize("TabHeader")
	b.Text = TabName
	b.TextSize = 16
	b.Typeface = Typeface.DEFAULT_BOLD
	b.Color = Colors.Black 
	b.TextColor = Colors.White 
	b.Tag = TabName
	mButtons.Put(TabName, b)
	
	ShowTabHeaders(act)
End Sub

'Shows the tab in TabName
'Throws an error if the tab doesn't exist
'
'Example: <code>th.ShowTab("Page 1")</code>
Public Sub ShowTab(TabName As String)
	If mTabs.ContainsKey(TabName) Then
		Dim pnl As Panel
		pnl = mTabs.Get(TabName)
		pnl.BringToFront 
		
		For i=0 To mButtons.Size-1
			Dim b As Button 
			b = mButtons.GetValueAt(i)
			b.Color = Colors.black
		Next
		
		Dim b As Button 
		b = mButtons.Get(TabName)
		b.Color = Colors.Blue 
		
		mCurrentTab = -1
		For i=0 To mButtons.Size-1
			Dim b As Button 
			b = mButtons.GetValueAt(i)
			If b.Tag=TabName Then
				mCurrentTab = i
				Exit
			End If
		Next
		
		If mCurrentTab>=0 Then	'let's try to raise the TabChanged callback
			If SubExists(mMeObj, mTabChangedCallBack) Then
				CallSub2(mMeObj, mTabChangedCallBack, TabName)
			End If
		End If
	Else
		Msgbox("Error. That tab doesn't exist", "Error")
	End If
End Sub

'Add a view to the tab content in TabName
'TabName is the tab we want to add the view to
'v is the view
'The position items are self explanatory
'
'Example: <code>Dim b1 as Button
'b1.Initialize("b1")
'b1.Text = "Hello. Click me"
'th.AddView("Page 1", b1, 0, 0, 200, 60)   'place it top left</code>
Public Sub AddView(TabName As String, v As View, Left As Int, Top As Int, Width As Int, Height As Int)
	If mTabs.ContainsKey(TabName) Then
		Dim pnl As Panel
		pnl = mTabs.Get(TabName)
		pnl.AddView(v, Left, Top, Width, Height)
	Else
		Msgbox("Error. Can't add this view because the Tab doesn't exist", "Unable to add")
	End If
End Sub

Private Sub TabHeader_Click
	Dim b As Button 
	b = Sender
	
	If mButtons.ContainsKey(b.Tag) Then
		ShowTab(b.Tag)
	End If
End Sub

Public Sub Panel_Touch(Action As Int, X As Float, Y As Float)
	If Action=mAct.ACTION_DOWN Then
		mStartX = X
	Else If Action=mAct.ACTION_UP Then
		If Abs(mStartX-X)>mMOVEMENTTHRESHOLD Then
			If mStartX>X Then	'left swipe
				If mCurrentTab=mButtons.Size-1 Then     'we need to show tab #0
					mCurrentTab = 0
					Dim b As Button 
					b = mButtons.GetValueAt(mCurrentTab)
					ShowTab(b.Tag)
				Else	'Show the tab to the right
					mCurrentTab = mCurrentTab + 1
					If mCurrentTab>=mButtons.Size Then mCurrentTab = 0
					Dim b As Button 
					b = mButtons.GetValueAt(mCurrentTab)
					ShowTab(b.Tag)
				End If
			Else		'Right swipe
				If mCurrentTab=0 Then
					mCurrentTab = mButtons.Size-1
					Dim b As Button
					b = mButtons.GetValueAt(mCurrentTab)
					ShowTab(b.Tag)
				Else
					mCurrentTab = mCurrentTab - 1
					If mCurrentTab<0 Then mCurrentTab = mButtons.Size-1
					Dim b As Button
					b = mButtons.GetValueAt(mCurrentTab)
					ShowTab(b.Tag)
				End If
			End If
		End If
	End If
End Sub

'Copy this code to your Activity and uncomment the line if you want to implement the swipe capability of the class
'Replace yourclassobject with the name of your cTabHost
Private Sub TabContent_Touch(Action As Int, X As Float, Y As Float)
	'yourclassobject.Panel_Touch(Action, X, Y)
End Sub